#include <Ivi.h>
#include <cvirte.h>		
#include <userint.h>
#include "ConfigStoreExample.h"
#include <Iviconfigserver.h>

static int panelHandle;

static IviConfigStoreHandle csHandle = 0;
static IviDriverSessionCollectionHandle driverSessionsHandle = 0;
static IviLogicalNameCollectionHandle logicalNamesHandle = 0;
	
static int InitializeLogicalNamesRingControl (IviConfigStoreHandle csHandle);
static int InitializeDriverSessionsRingControl (IviConfigStoreHandle csHandle);
static int InitializeCurrentDriverSessionDisplay (IviConfigStoreHandle csHandle);
static int DisposeAllHandles(IviConfigStoreHandle csHandle);




int __stdcall WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                       LPSTR lpszCmdLine, int nCmdShow)
{
	ViStatus error = VI_SUCCESS;
	
	if (InitCVIRTE (hInstance, 0, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "ConfigStoreExample.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (panelHandle);
	checkErr ( Ivi_GetConfigStoreHandle (&csHandle)); 
	checkErr ( IviConfig_GetConfigStoreLogicalNameCollection (csHandle, &logicalNamesHandle));
    checkErr ( IviConfig_GetConfigStoreDriverSessionCollection (csHandle, &driverSessionsHandle));
    checkErr ( InitializeLogicalNamesRingControl (csHandle));
	checkErr ( InitializeDriverSessionsRingControl (csHandle));
	checkErr ( InitializeCurrentDriverSessionDisplay (csHandle));
	RunUserInterface ();  
Error: 
	//Dispose Global Handles
	if (csHandle)
		IviConfig_DisposeHandle (csHandle);
	if (driverSessionsHandle)
		IviConfig_DisposeHandle (driverSessionsHandle);
	if (logicalNamesHandle)
		IviConfig_DisposeHandle (logicalNamesHandle);
	//Call function to Dispose all individual Logical Name and Driver Session Handle
	DisposeAllHandles (csHandle);
	DiscardPanel (panelHandle);
	return 0;
}

int CVICALLBACK OnSave (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViChar csLocation[1024];
	
	switch (event)
		{
		case EVENT_COMMIT:
			IviConfig_GetConfigStorePropertyViString (csHandle, IVICONFIG_VAL_CONFIG_STORE_MASTER_LOCATION,
                                                            1024, csLocation);
			IviConfig_Serialize (csHandle, csLocation);
			break;
		}
	return 0;
}

int CVICALLBACK OnExit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
		}
	return 0;
}

int CVICALLBACK OnChange (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	IviDriverSessionHandle driverSessionHandle = 0;
    IviLogicalNameHandle logicalNameHandle = 0;
    
    int i,j;
    
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlAttribute (PANEL, PANEL_LOGICALNAMES, ATTR_CTRL_INDEX, &i);
			GetCtrlAttribute (PANEL, PANEL_DRVSESSIONS, ATTR_CTRL_INDEX, &j);
			
			GetValueFromIndex (PANEL, PANEL_LOGICALNAMES, i, (int*)&logicalNameHandle);
			GetValueFromIndex (PANEL, PANEL_DRVSESSIONS, j, (int*)&driverSessionHandle);

			IviConfig_SetLogicalNameSessionReference (logicalNameHandle, driverSessionHandle);
			InitializeCurrentDriverSessionDisplay (csHandle);
			break;
		}
	return 0;
}

int CVICALLBACK OnLogicalNameChange (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_VAL_CHANGED:
			InitializeCurrentDriverSessionDisplay (csHandle);
			break;
		}
	return 0;
}

static int InitializeLogicalNamesRingControl (IviConfigStoreHandle csHandle)

{
	ViStatus error = VI_SUCCESS;
	ViInt32 logicalNameCount , i = 0;
	
	checkErr (IviConfig_GetLogicalNameCount (logicalNamesHandle,&logicalNameCount));
    
    for(i=1;i<=logicalNameCount;i++)
    {
    	IviLogicalNameHandle logicalNameHandle = 0;
    	ViChar name[512];
    	
    	IviConfig_GetLogicalNameItemByIndex (logicalNamesHandle, i, &logicalNameHandle);
    	IviConfig_GetLogicalNamePropertyViString (logicalNameHandle, IVICONFIG_VAL_LOGICAL_NAME_NAME,
                                                            512, name);
        InsertListItem (PANEL, PANEL_LOGICALNAMES, i-1, name, (int)logicalNameHandle);
    }
    
Error:
	return error;
}

static int InitializeDriverSessionsRingControl (IviConfigStoreHandle csHandle)

{
	ViStatus error = VI_SUCCESS;
	ViInt32 driverSessionCount , i = 0;
	
	checkErr (IviConfig_GetDriverSessionCount (driverSessionsHandle,&driverSessionCount));
    
    for(i=1;i<=driverSessionCount;i++)
    {
    	IviDriverSessionHandle driverSessionHandle = 0;
    	ViChar name[512];
    	
    	IviConfig_GetDriverSessionItemByIndex (driverSessionsHandle, i, &driverSessionHandle);
    	IviConfig_GetDriverSessionPropertyViString (driverSessionHandle, IVICONFIG_VAL_CONFIG_COMPONENT_NAME,
                                                            512, name);
		InsertListItem (PANEL, PANEL_DRVSESSIONS, i-1, name, (int)driverSessionHandle);
    }
    
Error:
	return error;
}

static int InitializeCurrentDriverSessionDisplay (IviConfigStoreHandle csHandle)

{
	int i=0;													 
	ViStatus error = VI_SUCCESS;
	IviDriverSessionHandle driverSessionHandle = 0;
    ViChar name[512], logicalName[512];
	
	SetCtrlVal (PANEL, PANEL_CURRENTSESSION, "");
	GetCtrlAttribute (PANEL, PANEL_LOGICALNAMES, ATTR_CTRL_INDEX, &i);
	GetLabelFromIndex (PANEL, PANEL_LOGICALNAMES, i, logicalName);
		
	checkErr (IviConfig_GetDriverSession (csHandle, logicalName, &driverSessionHandle));
	checkErr (IviConfig_GetDriverSessionPropertyViString (driverSessionHandle, IVICONFIG_VAL_CONFIG_COMPONENT_NAME,
                                                            512, name));
    SetCtrlVal (PANEL, PANEL_CURRENTSESSION, name);

Error:
	if (driverSessionHandle)
		IviConfig_DisposeHandle (driverSessionHandle);
	return error;
}

static int DisposeAllHandles(IviConfigStoreHandle csHandle)

{
	int i=0,lnCount=0,dsCount=0;
	
	IviDriverSessionHandle driverSessionHandle = 0;
	IviLogicalNameHandle logicalNameHandle = 0;
	
	GetNumListItems (PANEL, PANEL_LOGICALNAMES, &lnCount);
	
	for (i=0;i<lnCount;i++)
	{
		GetValueFromIndex (PANEL, PANEL_LOGICALNAMES, i, (int*)&logicalNameHandle);
		if (logicalNameHandle)
			IviConfig_DisposeHandle(logicalNameHandle);
	}
	
	GetNumListItems (PANEL, PANEL_DRVSESSIONS, &dsCount);
	
	for (i=0;i<dsCount;i++)
	{
		GetValueFromIndex (PANEL, PANEL_DRVSESSIONS, i, (int*)&driverSessionHandle);
		if (driverSessionHandle)
			IviConfig_DisposeHandle(driverSessionHandle);
	}
	
	return 0;
}
